use std::fs::File;
use std::io::Write;
use std::path::PathBuf;

use libpt::log::{debug, error, info, warn};
use serde::{Deserialize, Serialize};

use crate::error::Error;

#[derive(Serialize, Deserialize, Default, Debug)]
pub struct Store {
    dummy: String,
}

impl Store {
    pub fn projdir() -> Option<directories::ProjectDirs> {
        directories::ProjectDirs::from("de.cscherr", "Beatbär", "Beatbär")
    }

    pub fn genres(&self) -> Vec<crate::player::ui::entry::Entry> {
        vec![]
    }

    pub fn artists(&self) -> Vec<crate::player::ui::entry::Entry> {
        vec![]
    }

    pub fn playlists(&self) -> Vec<crate::player::ui::entry::Entry> {
        vec![]
    }

    pub fn songs(&self) -> Vec<crate::player::ui::entry::Entry> {
        vec![]
    }

    pub fn albums(&self) -> Vec<crate::player::ui::entry::Entry> {
        vec![]
    }

    pub fn save(&self) -> Result<(), Error> {
        info!("saving the store to file");
        debug!("Store: {self:#?}");
        if let Some(dirs) = Self::projdir() {
            let mut store_path: PathBuf = dirs.data_local_dir().into();
            store_path.push("store.msgpack");

            if !store_path.exists() {
                std::fs::create_dir_all(
                    store_path
                        .parent()
                        .expect("beatbär storefile has no parent????"),
                )?;
                warn!("The Beatbär store at '{}' does not exist. Creating it anew. This is normal if you start Beatbär for the first time.", store_path.as_path().to_string_lossy());
            } else if !store_path.is_file() {
                let e = Error::BadStoreFile(store_path);
                error!("{e}");
                return Err(e);
            }

            let mut file = File::options()
                .write(true)
                .append(false)
                .create(true)
                .truncate(true)
                .open(store_path)?;
            let repr = rmp_serde::to_vec(self)?;
            file.write_all(&repr)?;
            info!("store file was written");
            Ok(())
        } else {
            Err(Error::NoProjDir)
        }
    }

    pub fn load() -> Result<Self, Error> {
        info!("loading the store to file");
        if let Some(dirs) = Self::projdir() {
            let mut store_path: PathBuf = dirs.data_local_dir().into();
            store_path.push("store.msgpack");

            if !store_path.exists() {
                warn!(
                    "The Beatbär store at '{}' does not exist. Loading an empty store instead.",
                    store_path.as_path().to_string_lossy()
                );
                return Ok(Self::default());
            } else if !store_path.is_file() {
                let e = Error::BadStoreFile(store_path);
                error!("{e}");
                return Err(e);
            }

            let file = File::options().read(true).write(false).open(store_path)?;
            let store = rmp_serde::from_read(file)?;
            info!("store file was loaded");
            debug!("Store: {store:#?}");
            Ok(store)
        } else {
            let e = Error::NoProjDir;
            error!("{}", e);
            std::process::exit(1);
        }
    }
}
